/*
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

/*
 *    IntervalExtract.java
 *    Copyright (C) 2010 Julian Luengo
 *
 */
package complexityruleextraction;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.lang.reflect.Array;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Locale;

/**
 *
 * @author Julian Luengo Martin
 * @date September 2008-2013
 */
public class IntervalExtract {

    /* Flags to know which plotting is desired */
    boolean printEPScolor = true;
    boolean printEPSbn = false;
    boolean printPNG = true;
    
    /* Variables to control the automatic extraction method*/
    double diffThreshold = 10;  //deprecated. Not used anymore  
    double minGoodElementTest = 90;
    double minGoodIntervalTest = 90;
    double threshold = 8;
    
    /* Arrays to store the accuracy values of the method we are analyzing*/
    double[] accTrain;
    double[] accTest;
    double cmet[];
    String dataset[];
    int currentDC;
    
    /* Global statistics obtained from the data provided by the text file*/
    double globalTrainAvg = 0;
    double globalTestAvg = 0;
    double globalTrainStDev = 0;
    double globalTestStDev = 0;
    double globalDiff = 0;
    double globalMeanDiff = 0;
    ArrayList<Interval> intervals[] = (ArrayList<Interval>[]) Array.newInstance(ArrayList.class, 15);
    /**
     * Minimum percentage of support of the rule respect to the total number of
     * data sets
     */
    double minSupport = 0.15;
    /* These ArrayList serve for separate the data set found to be covered by the good
     and bad domains of competence respectively.*/
    ArrayList<String[]> globalGood = new ArrayList<String[]>();
    ArrayList<String[]> globalBad = new ArrayList<String[]>();
    NumberFormat nf = NumberFormat.getNumberInstance(Locale.ENGLISH);

    /**
     * Default constructor. It creates an object with the specified threshold
     * and minimum test value in order to be considered a good behavior element.
     *
     * @param threshold The threshold to be applied from now on
     * @param minTest The minimum test accuracy value obtained for a single data
     * set in order to constitute a good behavior element.
     */
    public IntervalExtract() {
        this.threshold = 8;
        this.minGoodElementTest = 90;
    }

    /**
     * Parametrized constructor. It creates an object with the specified
     * threshold and minimum test value in order to be considered a good
     * behavior element.
     *
     * @param threshold The threshold to be applied from now on
     * @param minTest The minimum test accuracy value obtained for a single data
     * set in order to constitute a good behavior element.
     */
    public IntervalExtract(double threshold, double minTest, boolean printEPScolor, boolean printEPSbn, boolean printPNG) {
        this.threshold = threshold;
        this.minGoodElementTest = minTest;

        this.printEPSbn = printEPSbn;
        this.printEPScolor = printEPScolor;
        this.printPNG = printPNG;
    }

    /**
     * From the given file, this method initializes all structures, and stores
     * the statisticals related to the current problem
     *
     * @param filename The File with the related complexity metrics and accuracy
     */
    public void initialize(String filename) {
        String data;
        String out[], aux[];
        data = Fichero.leeFichero(filename);
        out = data.split(System.getProperty("line.separator"));
        double squareTrain, squareTest;

        accTrain = new double[out.length];
        accTest = new double[out.length];
        cmet = new double[out.length];
        dataset = new String[out.length];
        squareTrain = squareTest = 0;
        globalTrainAvg = 0;
        globalTestAvg = 0;
        globalTrainStDev = 0;
        globalTestStDev = 0;
        globalDiff = 0;
        globalMeanDiff = 0;
        for (int i = 0; i < out.length; i++) {
            aux = out[i].split("\\t");
            dataset[i] = aux[0];
            cmet[i] = Double.valueOf(aux[1]);
            accTrain[i] = Double.valueOf(aux[2]);
            globalTrainAvg += accTrain[i];
            squareTrain += (accTrain[i] * accTrain[i]);
            accTest[i] = Double.valueOf(aux[3]);
            globalTestAvg += accTest[i];
            squareTest += (accTest[i] * accTest[i]);
            globalMeanDiff += accTrain[i] - accTest[i];
        }
        globalTrainAvg /= accTrain.length;
        globalTestAvg /= accTest.length;
        globalMeanDiff /= accTrain.length;
        globalMeanDiff = Math.max(globalMeanDiff, 0);

        globalTrainStDev = squareTrain / accTrain.length - (globalTrainAvg * globalTrainAvg);
        globalTrainStDev = Math.sqrt(globalTrainStDev);

        globalTestStDev = squareTest / accTest.length - (globalTestAvg * globalTestAvg);
        globalTestStDev = Math.sqrt(globalTestStDev);

        globalDiff = globalTrainAvg - globalTestAvg;

//        System.out.println(globalTrainAvg+"\t"+globalTestAvg);
//        System.out.println(globalTrainStDev+"\t"+globalTestStDev);
//
//        System.out.println(globalDiff+"\t"+globalMeanDiff);
        //Parameters which define a quality point
        //static definition
        //minGoodElementTest = 90.0;
        minGoodIntervalTest = minGoodElementTest;
        //dynamic definition
//        minGoodElementTest = globalTrainAvg+5;
//        minGoodIntervalTest = globalTestAvg+5;

        //diffThreshold = 10;

        nf.setGroupingUsed(false);     // don't group by threes
        nf.setMaximumFractionDigits(2);
        nf.setMinimumFractionDigits(2);

        for (int i = 0; i < 15; i++) {
            intervals[i] = new ArrayList<Interval>();
        }
    }

    /**
     * From the given table, this method initializes all structures, and stores
     * the statisticals related to the current problem
     *
     * @param datasets The table with the data complexity measures for each data
     * set and its training/test accuracy values
     * @param dcMeasure the data complexity measure which will be considered
     */
    public void initialize(ArrayList<dcTrainingTest> datasets, int dcMeasure) {
        dcTrainingTest dctt;
        double squareTrain, squareTest;

        accTrain = new double[datasets.size()];
        accTest = new double[datasets.size()];
        cmet = new double[datasets.size()];
        dataset = new String[datasets.size()];
        squareTrain = squareTest = 0;
        globalTrainAvg = 0;
        globalTestAvg = 0;
        globalTrainStDev = 0;
        globalTestStDev = 0;
        globalDiff = 0;
        globalMeanDiff = 0;
        for (int i = 0; i < datasets.size(); i++) {
            dctt = datasets.get(i);
            dataset[i] = dctt.getDatasetName();
            cmet[i] = dctt.getValues()[dcMeasure];
            accTrain[i] = dctt.getTrainingAccuracy();
            globalTrainAvg += accTrain[i];
            squareTrain += (accTrain[i] * accTrain[i]);
            accTest[i] = dctt.getTestAccuracy();
            globalTestAvg += accTest[i];
            squareTest += (accTest[i] * accTest[i]);
            globalMeanDiff += accTrain[i] - accTest[i];
        }
        globalTrainAvg /= accTrain.length;
        globalTestAvg /= accTest.length;
        globalMeanDiff /= accTrain.length;
        globalMeanDiff = Math.max(globalMeanDiff, 0);

        globalTrainStDev = squareTrain / accTrain.length - (globalTrainAvg * globalTrainAvg);
        globalTrainStDev = Math.sqrt(globalTrainStDev);

        globalTestStDev = squareTest / accTest.length - (globalTestAvg * globalTestAvg);
        globalTestStDev = Math.sqrt(globalTestStDev);

        globalDiff = globalTrainAvg - globalTestAvg;

//        System.out.println(globalTrainAvg+"\t"+globalTestAvg);
//        System.out.println(globalTrainStDev+"\t"+globalTestStDev);
//
//        System.out.println(globalDiff+"\t"+globalMeanDiff);
        //Parameters which define a quality point
        //static definition
        //minGoodElementTest = 90.0;
        minGoodIntervalTest = minGoodElementTest;
        //dynamic definition
//        minGoodElementTest = globalTrainAvg+5;
//        minGoodIntervalTest = globalTestAvg+5;

        //diffThreshold = 5;

        nf.setGroupingUsed(false);     // don't group by threes
        nf.setMaximumFractionDigits(2);
        nf.setMinimumFractionDigits(2);

        for (int i = 0; i < 15; i++) {
            intervals[i] = new ArrayList<Interval>();
        }
    }

    /**
     * This method runs the process of finding and showing intervals of a given
     * Complexity metric based on the Accuracy in train test.
     *
     * @param filename The path to the file with the Cmet. and Accuracy values
     */
    public void process(String filename) {
        //First, we process the file with the Train/Test accuracy
        //This file has the following format:
        //Data_set1 Cmet1 train1    test1
        //Data_set2 Cmet2 train1    test2
        //Data_set3 Cmet3 train1    test3
        //...
        //the Data_seti, Cmeti, train_i and test_i values are separated by a tab character (\t)
        //The train and test values MUST be sorted by the Data complexity measure
        //to obtain meaningful results
        initialize(filename);

        System.out.println("File: " + filename);
        System.out.println("Global mean Train Accuracy: " + globalTrainAvg);
        System.out.println("Global mean Test Accuracy: " + globalTestAvg + "\n");
        System.out.println("Quality Intervals:");
        System.out.print("Range");
        System.out.print("\tSupport");
        System.out.print("\tmeanTrainAcc");
        System.out.print("\tmeanTestAcc");
        System.out.print("\tstdTrain");
        System.out.println("\tstdTest");
        extendGoodIntervalByMean();
        System.out.println("\nBad Intervals:");
        System.out.print("Range");
        System.out.print("\tSupport");
        System.out.print("\tmeanTrainAcc");
        System.out.print("\tmeanTestAcc");
        System.out.print("\tstdTrain");
        System.out.println("\tstdTest");
        extendBadIntervalByMean();
    }

    /**
     * Process a list of text files with the complexity metrics and the accuracy
     * results.
     *
     * @param list The text file with the path to the different files.
     */
    public void processList(String list) {
        String data;
        String sp[], out[], aux[];
        data = Fichero.leeFichero(list);
        sp = data.split(System.getProperty("line.separator"));
        int maxSize;

        System.out.print("\tRange");
        System.out.print("\tSupport");
        System.out.print("\tmeanTrainAcc");
        System.out.print("\tTrain diff.");
        System.out.print("\tmeanTestAcc");
        System.out.print("\tTest diff.");
        System.out.print("\tstdTrain");
        System.out.println("\tstdTest");
        for (int i = 0; i < sp.length; i++) {
            currentDC = i;
            processNoVerbose(sp[i]);
        }
        //Print the good rules' itemsets to file
        maxSize = -1;
        for (int i = 0; i < globalGood.size(); i++) {
            maxSize = Math.max(globalGood.get(i).length, maxSize);
        }
        data = "";
        for (int j = 0; j < maxSize; j++) {
            for (int i = 0; i < globalGood.size(); i++) {
                if (globalGood.get(i).length > j) {
                    data += globalGood.get(i)[j];
                }
                data += "\t";
            }
            data += System.getProperty("line.separator");
        }
        Fichero.escribeFichero("good.txt", data);

        //Print the bad rules' itemsets to file
        maxSize = -1;
        for (int i = 0; i < globalBad.size(); i++) {
            maxSize = Math.max(globalBad.get(i).length, maxSize);
        }
        data = "";
        for (int j = 0; j < maxSize; j++) {
            for (int i = 0; i < globalBad.size(); i++) {
                if (globalBad.get(i).length > j) {
                    data += globalBad.get(i)[j];
                }
                data += "\t";
            }
            data += System.getProperty("line.separator");
        }
        Fichero.escribeFichero("bad.txt", data);
    }

    /**
     * Process a table from a text file with the complexity metrics and the
     * accuracy results separated by tabs.
     *
     * @param list The text file with the path to the text file with the table.
     */
    public void processTable(String list) {
        String data;
        String sp[], out[], aux[];
        data = Fichero.leeFichero(list);
        sp = data.split(System.getProperty("line.separator"));
        if (sp.length <= 1) {
            sp = data.split("\\n");
        }
        int maxSize;
        ArrayList<dcTrainingTest> datasets = new ArrayList<dcTrainingTest>();

        System.out.print("\tRange");
        System.out.print("\tSupport");
        System.out.print("\tmeanTrainAcc");
        System.out.print("\tTrain diff.");
        System.out.print("\tmeanTestAcc");
        System.out.print("\tTest diff.");
        System.out.print("\tstdTrain");
        System.out.println("\tstdTest");
        for (int i = 0; i < sp.length; i++) {
            datasets.add(processTableLine(sp[i]));
        }

        //process each data complexity measure
        for (int i = 0; i < 15; i++) {
            currentDC = i;
            processNoVerbose(datasets, i);
        }
        //Print the good rules' itemsets to file
        maxSize = -1;
        for (int i = 0; i < globalGood.size(); i++) {
            maxSize = Math.max(globalGood.get(i).length, maxSize);
        }
        data = "";
        for (int j = 0; j < maxSize; j++) {
            for (int i = 0; i < globalGood.size(); i++) {
                if (globalGood.get(i).length > j) {
                    data += globalGood.get(i)[j];
                }
                data += "\t";
            }
            data += System.getProperty("line.separator");
        }
        Fichero.escribeFichero("good.txt", data);

        //Print the bad rules' itemsets to file
        maxSize = -1;
        for (int i = 0; i < globalBad.size(); i++) {
            maxSize = Math.max(globalBad.get(i).length, maxSize);
        }
        data = "";
        for (int j = 0; j < maxSize; j++) {
            for (int i = 0; i < globalBad.size(); i++) {
                if (globalBad.get(i).length > j) {
                    data += globalBad.get(i)[j];
                }
                data += "\t";
            }
            data += System.getProperty("line.separator");
        }
        Fichero.escribeFichero("bad.txt", data);
    }

    public static dcTrainingTest processTableLine(String line) {
        String out[];
        dcTrainingTest dc = new dcTrainingTest();

        out = line.split("\\t");
        dc.setName(out[0]);
        dc.setF1(Double.valueOf(out[1]));
        dc.setF2(Double.valueOf(out[2]));
        dc.setF3(Double.valueOf(out[3]));
        dc.setN1(Double.valueOf(out[4]));
        dc.setN2(Double.valueOf(out[5]));
        dc.setN3(Double.valueOf(out[6]));
        dc.setN4(Double.valueOf(out[7]));
        dc.setL1(Double.valueOf(out[8]));
        dc.setL2(Double.valueOf(out[9]));
        dc.setL3(Double.valueOf(out[10]));
        dc.setT1(Double.valueOf(out[11]));
        dc.setT2(Double.valueOf(out[12]));

        if (out.length > 15) {
            dc.setNPos(Double.valueOf(out[13]));
            dc.setNNeg(Double.valueOf(out[14]));
            dc.setIR(Double.valueOf(out[15]));

            dc.setAccuracy(Double.valueOf(out[16]), Double.valueOf(out[17]));
        } else {
            dc.setAccuracy(Double.valueOf(out[13]), Double.valueOf(out[14]));
        }


        return dc;

    }

    /**
     * This method runs the process of finding and showing intervals of a given
     * Complexity metric based on the Accuracy in train test. It performs this
     * task silently.
     *
     * @param filename The path to the file with the Cmet. and Accuracy values.
     */
    public void processNoVerbose(String filename) {
        String data;
        String out[], aux[];
        ArrayList<Interval> ivals;
        String[] localDataSet;

        //First, we process the file with the Train/Test accuracy
        //This file has the following format:
        //Data_set1 Cmet_i^1 train1    test1
        //Data_set2 Cmet_i^2 train1    test2
        //Data_set3 Cmet_i^3 train1    test3
        //...
        //the Data_set_i, Cmet_i, train_i and test_i values are separated by a tab character (\t)
        //The train and test values MUST be sorted by the Data complexity measure in order
        //to obtain meaningful results
        initialize(filename);

        System.out.print(filename + "good\t");
        ivals = extendGoodIntervalByMean();
        if (ivals.size() == 0) {
            System.out.println();
        } else {
            for (int i = 0; i < ivals.size(); i++) {
                localDataSet = new String[ivals.get(i).length() + 1];
                localDataSet[0] = filename + "-good-" + i;
                for (int j = ivals.get(i).lowerBound, k = 1; j <= ivals.get(i).upperBound; j++, k++) {
                    localDataSet[k] = dataset[j];
                }
                globalGood.add(localDataSet);
            }
        }
        System.out.print(filename + "bad\t");
        ivals = extendBadIntervalByMean();
        if (ivals.size() == 0) {
            System.out.println();
        } else {
            for (int i = 0; i < ivals.size(); i++) {
                localDataSet = new String[ivals.get(i).length() + 1];
                localDataSet[0] = filename + "-bad-" + i;
                for (int j = ivals.get(i).lowerBound, k = 1; j <= ivals.get(i).upperBound; j++, k++) {
                    localDataSet[k] = dataset[j];
                }
                globalBad.add(localDataSet);
            }
        }
    }

    /**
     * This method runs the process of finding and showing intervals of a given
     * Complexity metric based on the Accuracy in train test. It performs this
     * task silently.
     *
     * @param datasets The table with the Cmet. and Accuracy values.
     * @param dcMeasure the data complexity measure considered
     */
    public void processNoVerbose(ArrayList<dcTrainingTest> datasets, int dcMeasure) {
        String data;
        String out[], aux[];
        ArrayList<Interval> ivals;
        Interval ival;
        String[] localDataSet;

        //First, we process the file with the Train/Test accuracy
        //This file has the following format:
        //Data_set1 F1  F2  F3  N1  N2  N3  N4  L1  L2  L3  T1  T2 NPos Nneg IR train1    test1
        //Data_set2 F1  F2  F3  N1  N2  N3  N4  L1  L2  L3  T1  T2 NPos Nneg IR train1    test2
        //Data_set3 F1  F2  F3  N1  N2  N3  N4  L1  L2  L3  T1  T2 NPos Nneg IR train1    test3
        //...
        //the Data_set_i, Cmet_i, train_i and test_i values are separated by a tab character (\t)
        //The train and test values MUST be sorted by the Data complexity measure
        //indicated by 'dcMeasure' in order
        //to obtain meaningful results

        dcTrainingTest.compareBy(dcMeasure);
        Collections.sort(datasets);
        initialize(datasets, dcMeasure);

        System.out.print(dcTrainingTest.dcNames[dcMeasure] + "-good\t");
        ivals = extendGoodIntervalByMean();
        if (ivals.size() == 0) {
            System.out.println();
        } else {
            for (int i = 0; i < ivals.size(); i++) {
                localDataSet = new String[ivals.get(i).length() + 1];
                localDataSet[0] = dcTrainingTest.dcNames[dcMeasure] + "-good-" + i;
                for (int j = ivals.get(i).lowerBound, k = 1; j <= ivals.get(i).upperBound; j++, k++) {
                    localDataSet[k] = dataset[j];
                }
                globalGood.add(localDataSet);
            }
        }
        System.out.print(dcTrainingTest.dcNames[dcMeasure] + "-bad\t");
        ivals = extendBadIntervalByMean();
        if (ivals.size() == 0) {
            System.out.println();
        } else {
            for (int i = 0; i < ivals.size(); i++) {
                localDataSet = new String[ivals.get(i).length() + 1];
                localDataSet[0] = dcTrainingTest.dcNames[dcMeasure] + "-bad-" + i;
                for (int j = ivals.get(i).lowerBound, k = 1; j <= ivals.get(i).upperBound; j++, k++) {
                    localDataSet[k] = dataset[j];
                }
                globalBad.add(localDataSet);
            }
        }

        //print the figures with the intervals
        createDataFile("data.txt");
        ArrayList<Interval> bounds = new ArrayList<Interval>();
        for (int i = 0; i < 15; i++) {
            for (int j = 0; j < intervals[i].size(); j++) {
                bounds.add(intervals[i].get(j));

            }
            if (printPNG) {
                plotIntervalsPNG(dcTrainingTest.dcNames[currentDC] + ".png", dcTrainingTest.dcNames[currentDC], "data.txt", bounds);
            }
            if (this.printEPScolor) {
                plotIntervalsEPS_color(dcTrainingTest.dcNames[currentDC] + "-color.eps", dcTrainingTest.dcNames[currentDC], "data.txt", bounds);
            }
            if (this.printEPSbn) {
                plotIntervalsEPS_bn(dcTrainingTest.dcNames[currentDC] + "-bn.eps", dcTrainingTest.dcNames[currentDC], "data.txt", bounds);
            }
        }
    }

    /**
     * Search for an interval by means of looking for a quality point and adding
     * the adjacent points while the resulting interval mean is above certain
     * threshold.
     *
     * @return the obtained intervals.
     */
    public ArrayList<Interval> extendGoodIntervalByMean() {
        int position = 0;
        Interval ival, biggerIval;
        ArrayList<Interval> ivals = new ArrayList<Interval>();

        do {
            position = look4qualitypoint(position);
            if (position != -1) {
                ival = extendGoodIntervalByMean(position);

                if (ival != null && ival.upperBound - ival.lowerBound + 1 >= accTrain.length * minSupport) {
                    ivals.add(ival);
                    //printInterval(ival);
                } else {
                    /*
                     Perhaps the interval would be good as well, under the following
                     precepts:
                     * The union with the previous interval is also a good interval
                     * The previous interval and the current one are very proximous
                     This situation could be caused by a set of "not enough good"
                     points, which split a big interval in two sub-intervals, and
                     one of them is not big enough in support.
                     */
                    if (ivals.size() > 0 && ival.upperBound - ival.lowerBound + 1 < accTrain.length * minSupport
                            && ival.lowerBound - ivals.get(ivals.size() - 1).upperBound < 5
                            && isQualityInterval(new Interval(ivals.get(ivals.size() - 1).lowerBound, ival.upperBound))) {
                        biggerIval = new Interval(ivals.get(ivals.size() - 1).lowerBound, ival.upperBound);
                        ivals.remove(ivals.size() - 1);
                        ivals.add(biggerIval);
                    } else {
                        //add it either way, as small intervals will be dropped later by mergeOverlappedIntervals()

                        ivals.add(ival);
                    }

                }
                //more exhaustive
                //position++;

                //less overlapping between intervals
                position = ival.upperBound + 1;
                while (position < accTrain.length - 1 && cmet[position] == cmet[position + 1]) {
                    position++;
                }
            }

            //System.out.println("Posicion: "+position);
        } while (position != -1 && position < accTrain.length);
        mergeOverlappedIntervals(ivals, true);
        for (int i = 0; i < ivals.size(); i++) {

            ivals.get(i).lowerCMet = (ivals.get(i).lowerBound == 0) ? 0 : cmet[ivals.get(i).lowerBound];
            ivals.get(i).upperCMet = (ivals.get(i).upperBound == accTrain.length - 1) ? Double.POSITIVE_INFINITY : cmet[ivals.get(i).upperBound];
            printInterval(ivals.get(i));

            ivals.get(i).goodInterval = true;
            intervals[currentDC].add(ivals.get(i));
            //createDataFile("data.txt");
            //plotInterval(dcTrainingTest.dcNames[currentDC]+"-good.png", dcTrainingTest.dcNames[currentDC]+"-good", "data.txt", ivals.get(i).lowerBound, ivals.get(i).upperBound);
        }

        return ivals;

    }

    /**
     * Searches for all the posible bad intervals by means of looking for a
     * quality point and adding the adjacent points while the resulting interval
     * mean is under certain threshold.
     *
     * @return the obtained intervals
     */
    public ArrayList<Interval> extendBadIntervalByMean() {
        int position = 0;
        Interval ival, biggerIval;
        ArrayList<Interval> ivals = new ArrayList<Interval>();

        do {
            position = look4Badpoint(position);
            if (position != -1) {
                ival = extendBadIntervalByMean(position);

                if (ival != null && ival.upperBound - ival.lowerBound + 1 >= accTrain.length * minSupport) {
                    ivals.add(ival);
                    //printInterval(ival);
                } else {
                    /*
                     Perhaps the interval would be bad as well, under the following
                     precepts:
                     * The union with the previous interval is also a bad interval
                     * The previous interval and the current one are very proximous
                     This situation could be caused by a set of "not enough bad"
                     points, which aplit a big interval in two sub-intervals, and
                     one of them is not big enough in support.
                     */
                    if (ivals.size() > 0 && ival.upperBound - ival.lowerBound + 1 < accTrain.length * minSupport
                            && ival.lowerBound - ivals.get(ivals.size() - 1).upperBound < 5
                            && isBadInterval(new Interval(ivals.get(ivals.size() - 1).lowerBound, ival.upperBound))) {
                        //System.out.println(ival.upperBound-ival.lowerBound+1);
                        biggerIval = new Interval(ivals.get(ivals.size() - 1).lowerBound, ival.upperBound);
                        ivals.remove(ivals.size() - 1);
                        ivals.add(biggerIval);
                    } else {
                        //add it either way, as small intervals will be dropped later by mergeOverlappedIntervals()

                        ivals.add(ival);
                    }


                }
                //more exhaustive
                //position++;

                //less overlapping between intervals
                position = ival.upperBound + 1;
                while (position < accTrain.length - 1 && cmet[position] == cmet[position + 1]) {
                    position++;
                }
            }

            //System.out.println("Posicion: "+position);
        } while (position != -1 && position < accTrain.length);
        mergeOverlappedIntervals(ivals, false);
        for (int i = 0; i < ivals.size(); i++) {

            ivals.get(i).lowerCMet = (ivals.get(i).lowerBound == 0) ? 0 : cmet[ivals.get(i).lowerBound];
            ivals.get(i).upperCMet = (ivals.get(i).upperBound == accTrain.length - 1) ? Double.POSITIVE_INFINITY : cmet[ivals.get(i).upperBound];
            printInterval(ivals.get(i));

            ivals.get(i).goodInterval = false;
            intervals[currentDC].add(ivals.get(i));
            //createDataFile("data.txt");
            //plotInterval(dcTrainingTest.dcNames[currentDC]+"-bad.png", dcTrainingTest.dcNames[currentDC]+"-bad", "data.txt", ivals.get(i).lowerBound, ivals.get(i).upperBound);
        }

        return ivals;
    }

    /**
     * Try to extend one initial point to a quality interval
     *
     * @param initialPosition Initial quality point
     * @return The resulting interval, if any
     */
    public Interval extendGoodIntervalByMean(int initialPosition) {
        boolean left, right, end;
        double accLeft, accRight, accBoth;
        Interval ival = new Interval(initialPosition, initialPosition);
        Interval bothIval, leftIval, rightIval;

        left = right = false;
        //extend the interval in both directions
        end = false;
        while (!end && isQualityInterval(ival) && ival.lowerBound >= 0 && ival.upperBound <= accTrain.length - 1
                && accTest[ival.lowerBound] > minGoodIntervalTest && accTest[ival.upperBound] > minGoodIntervalTest) {
            //first in both sides
            ival.lowerBound--;
            ival.upperBound++;
            left = right = true;

            //test if the interval limits have the same Cmet value.
            //If so, then we must extend such interval to cover all
            //points with such cmet value (since we establish the interval
            //from the Cmet value, not the point position)
            if (ival.lowerBound > 0 && cmet[ival.lowerBound] == cmet[ival.lowerBound - 1]) {
                //let's expand in both directions while the cmet value
                //is the same
                while (ival.lowerBound > 0 && cmet[ival.lowerBound] == cmet[ival.lowerBound - 1]) {
                    ival.lowerBound--;
                }
//            if(ival.lowerBound!=0)
//                ival.lowerBound++;
            }
            if (ival.upperBound < accTrain.length - 1 && cmet[ival.upperBound] == cmet[ival.upperBound + 1]) {
                while (ival.upperBound < accTrain.length - 1 && cmet[ival.upperBound + 1] == cmet[ival.upperBound]) {
                    ival.upperBound++;
                }
//            if(ival.upperBound!=accTrain.length-1)
//                ival.upperBound--;
            }

            //check the limits
            if (ival.lowerBound < 0) {
                ival.lowerBound = 0;
                left = false;
            }
            if (ival.upperBound > accTrain.length - 1) {
                ival.upperBound = accTrain.length - 1;
                right = false;
            }
            if (!left && !right) {
                end = true;
            }
        }
        if (right) {
            ival.upperBound--;
            while (initialPosition < ival.upperBound && ival.upperBound > 0 && cmet[ival.upperBound + 1] == cmet[ival.upperBound]) {
                ival.upperBound--;
            }
        }
        if (left) {
            ival.lowerBound++;
            while (ival.lowerBound < initialPosition && ival.lowerBound < accTrain.length - 1 && cmet[ival.lowerBound] == cmet[ival.lowerBound - 1]) {
                ival.lowerBound++;
            }
        }
        accBoth = intervalMeanTrainAccuracy(ival);
        bothIval = new Interval(ival);


        //try to extend the interval in each direction
        //first, to the right (upper) side
        end = false;
        while (!end && isQualityInterval(ival) && ival.upperBound <= accTrain.length - 1
                && accTest[ival.upperBound] > minGoodIntervalTest) {
            ival.upperBound++;
            right = true;

            //test if the interval limits have the same Cmet value.
            //If so, then we must extend such interval to cover all
            //points with such cmet value (since we establish the interval
            //from the Cmet value, not the point position)
            if (ival.upperBound < accTrain.length - 1 && cmet[ival.upperBound] == cmet[ival.upperBound + 1]) {
                while (ival.upperBound < accTrain.length - 1 && cmet[ival.upperBound + 1] == cmet[ival.upperBound]) {
                    ival.upperBound++;
                }
//            if(ival.upperBound!=accTrain.length-1)
//                ival.upperBound--;
            }


            //check the limits
            if (ival.upperBound > accTrain.length - 1) {
                ival.upperBound = accTrain.length - 1;
                right = false;
                end = true;
            }
        }
        if (right) {
            ival.upperBound--;
            while (initialPosition < ival.upperBound && ival.upperBound > 0 && cmet[ival.upperBound + 1] == cmet[ival.upperBound]) {
                ival.upperBound--;
            }
        }
        accLeft = intervalMeanTrainAccuracy(ival);
        leftIval = new Interval(ival);


        //second, to the left (lower) side
        ival = new Interval(bothIval);
        end = false;
        while (!end && isQualityInterval(ival) && ival.lowerBound >= 0
                && accTest[ival.lowerBound] > minGoodIntervalTest) {
            ival.lowerBound--;
            left = true;

            if (ival.lowerBound > 0 && cmet[ival.lowerBound] == cmet[ival.lowerBound - 1]) {
                //let's expand in both directions while the cmet value
                //is the same
                while (ival.lowerBound > 0 && cmet[ival.lowerBound] == cmet[ival.lowerBound - 1]) {
                    ival.lowerBound--;
                }
//            if(ival.lowerBound!=0)
//                ival.lowerBound++;
            }

            //check the limits
            if (ival.lowerBound < 0) {
                ival.lowerBound = 0;
                left = false;
                end = true;
            }
        }
        if (left) {
            ival.lowerBound++;
            while (ival.lowerBound < initialPosition && ival.lowerBound < accTrain.length - 1 && cmet[ival.lowerBound] == cmet[ival.lowerBound - 1]) {
                ival.lowerBound++;
            }
        }
        accRight = intervalMeanTrainAccuracy(ival);
        rightIval = new Interval(ival);

        //look for the most convenient interval (i.e. the largest one)
        ival = rightIval;
        if (leftIval.length() > rightIval.length()) {
            ival = leftIval;
        }





        if (isQualityInterval(ival)) {
            return ival;
        }
        return (new Interval(initialPosition, initialPosition));
    }

    /**
     * Try to extend one initial point to a low quality interval
     *
     * @param initialPosition Initial low quality point
     * @return The resulting interval, if any
     */
    public Interval extendBadIntervalByMean(int initialPosition) {
        boolean left, right, end;
        double accLeft, accRight, accBoth;
        Interval ival = new Interval(initialPosition, initialPosition);
        Interval bothIval, leftIval, rightIval;

        left = right = false;
        //extend the interval in both directions
        end = false;
        while (!end && isBadInterval(ival) && ival.lowerBound >= 0 && ival.upperBound <= accTrain.length - 1) {
            //first in both sides
            ival.lowerBound--;
            ival.upperBound++;
            left = right = true;

            //test if the interval limits have the same Cmet value.
            //If so, then we must extend such interval to cover all
            //points with such cmet value (since we establish the interval
            //from the Cmet value, not the point position)
            if (ival.lowerBound > 0 && cmet[ival.lowerBound] == cmet[ival.lowerBound - 1]) {
                //let's expand in both directions while the cmet value
                //is the same
                while (ival.lowerBound > 0 && cmet[ival.lowerBound] == cmet[ival.lowerBound - 1]) {
                    ival.lowerBound--;
                }
//            if(ival.lowerBound!=0)
//                ival.lowerBound++;
            }
            if (ival.upperBound < accTrain.length - 1 && cmet[ival.upperBound] == cmet[ival.upperBound + 1]) {
                while (ival.upperBound < accTrain.length - 1 && cmet[ival.upperBound + 1] == cmet[ival.upperBound]) {
                    ival.upperBound++;
                }
//            if(ival.upperBound!=accTrain.length-1)
//                ival.upperBound--;
            }

            //check the limits
            if (ival.lowerBound < 0) {
                ival.lowerBound = 0;
                left = false;
            }
            if (ival.upperBound > accTrain.length - 1) {
                ival.upperBound = accTrain.length - 1;
                right = false;
            }
            if (!right && !left) {
                end = true;
            }
        }
        if (right) {
            ival.upperBound--;
            while (initialPosition < ival.upperBound && ival.upperBound > 0 && cmet[ival.upperBound + 1] == cmet[ival.upperBound]) {
                ival.upperBound--;
            }
        }
        if (left) {
            ival.lowerBound++;
            while (ival.lowerBound < initialPosition && ival.lowerBound < accTrain.length - 1 && cmet[ival.lowerBound] == cmet[ival.lowerBound - 1]) {
                ival.lowerBound++;
            }
        }
        accBoth = intervalMeanTrainAccuracy(ival);
        bothIval = new Interval(ival);


        //try to extend the interval in each direction
        //first, to the right (upper) side
        end = false;
        while (!end && isBadInterval(ival) && ival.upperBound <= accTrain.length - 1) {
            ival.upperBound++;
            right = true;

            if (ival.upperBound < accTrain.length - 1 && cmet[ival.upperBound] == cmet[ival.upperBound + 1]) {
                while (ival.upperBound < accTrain.length - 1 && cmet[ival.upperBound + 1] == cmet[ival.upperBound]) {
                    ival.upperBound++;
                }
//            if(ival.upperBound!=accTrain.length-1)
//                ival.upperBound--;
            }

            //check the limits
            if (ival.upperBound > accTrain.length - 1) {
                ival.upperBound = accTrain.length - 1;
                right = false;
                end = true;
            }
        }
        if (right) {
            ival.upperBound--;
            while (initialPosition < ival.upperBound && ival.upperBound > 0 && cmet[ival.upperBound + 1] == cmet[ival.upperBound]) {
                ival.upperBound--;
            }
        }
        accLeft = intervalMeanTrainAccuracy(ival);
        leftIval = new Interval(ival);


        //second, to the left (lower) side
        end = false;
        ival = new Interval(bothIval);
        while (!end && isBadInterval(ival) && ival.lowerBound >= 0) {
            ival.lowerBound--;
            left = true;

            if (ival.lowerBound > 0 && cmet[ival.lowerBound] == cmet[ival.lowerBound - 1]) {
                //let's expand in both directions while the cmet value
                //is the same
                while (ival.lowerBound > 0 && cmet[ival.lowerBound] == cmet[ival.lowerBound - 1]) {
                    ival.lowerBound--;
                }
//            if(ival.lowerBound!=0)
//                ival.lowerBound++;
            }

            //check the limits
            if (ival.lowerBound < 0) {
                ival.lowerBound = 0;
                left = false;
                end = true;
            }
        }
        if (left) {
            ival.lowerBound++;
            while (ival.lowerBound < initialPosition && ival.lowerBound < accTrain.length - 1 && cmet[ival.lowerBound] == cmet[ival.lowerBound - 1]) {
                ival.lowerBound++;
            }
        }
        accRight = intervalMeanTrainAccuracy(ival);
        rightIval = new Interval(ival);

        //look for the most convenient interval (i.e. the largest one)
        ival = rightIval;
        if (leftIval.length() > rightIval.length()) {
            ival = leftIval;
        }

        if (isBadInterval(ival)) {
            return ival;
        }
        return (new Interval(initialPosition, initialPosition));
    }

    /**
     * Search for a quality point in the vector (return the first one)
     *
     * @param from initial position for searching
     * @return the first quality point found
     */
    public int look4qualitypoint(int from) {

        for (int i = from; i < accTrain.length; i++) {
            if (isQualityPoint(accTrain[i], accTest[i])) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Search for a low quality point in the vector (return the first one)
     *
     * @param from initial position for searching
     * @return the first low quality point found
     */
    public int look4Badpoint(int from) {

        for (int i = from; i < accTrain.length; i++) {
            if (isBadPoint(accTrain[i], accTest[i])) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Test if the point is a quality one
     *
     * @param train the train accuracy of the point
     * @param test the test accuracy of the point
     * @return True if it is quality point, false otherwise
     */
    public boolean isQualityPoint(double train, double test) {
        double difference;

        difference = train - test;
        if (difference <= globalMeanDiff) {
            if (test >= minGoodElementTest) {
                return true;
            }
        }

        return false;
    }

    /**
     * Test if the point is a quality one
     *
     * @param pos The position of the reference point
     * @return True if it is quality point, false otherwise
     */
    public boolean isQualityPoint(int pos) {
        return isQualityPoint(accTrain[pos], accTest[pos]);
    }

    /**
     * Test if the point is a low quality one
     *
     * @param train the train accuracy of the point
     * @param test the test accuracy of the point
     * @return True if it is alow quality point, false otherwise
     */
    public boolean isBadPoint(double train, double test) {
        double difference;

        difference = train - test;
        if (test < minGoodElementTest) {
            if (difference >= globalMeanDiff /*|| train <= globalTrainAvg - 10*/) {
                return true;
            }
        }

        return false;
    }

    /**
     * Test if the interval is a quality one
     *
     * @param ival The interval to be tested
     * @return if it is quality interval
     */
    public boolean isQualityInterval(Interval ival) {
        double avgTrain, avgTest, avgDiff;

        avgTrain = intervalMeanTrainAccuracy(ival);
        avgTest = intervalMeanTestAccuracy(ival);
        avgDiff = Math.max(intervalMeanTrainTestDifference(ival), 0);

        if ((avgTest >= (globalTestAvg + threshold)) && (avgDiff) <= globalMeanDiff) { // original, average performance
            //if ((avgTest >= (minGoodElementTest + threshold)) && (avgDiff) <= globalMeanDiff) {
            return true;
        }
        return false;
    }

    /**
     * Test if the interval is a low quality one
     *
     * @param ival The interval to be tested
     * @return if it is low quality interval
     */
    public boolean isBadInterval(Interval ival) {
        double avgTrain, avgTest, avgDiff;

        avgTrain = intervalMeanTrainAccuracy(ival);
        avgTest = intervalMeanTestAccuracy(ival);
        avgDiff = Math.max(intervalMeanTrainTestDifference(ival), 0);

        if ((avgDiff) >= globalMeanDiff && avgTest < (globalTestAvg - threshold)) { //original
            //if ((avgDiff) >= globalMeanDiff && avgTest < (minGoodElementTest - threshold)) {
            return true;
        }
        return false;
    }

    /**
     * Computes the average train accuracy of the given interval
     *
     * @param ival The reference interval of accuracy points
     * @return the mean train accuracy of the interval
     */
    public double intervalMeanTrainAccuracy(Interval ival) {
        double avg = 0;

        for (int i = ival.lowerBound; i <= ival.upperBound; i++) {
            avg += accTrain[i];
        }
        avg /= (double) (ival.upperBound - ival.lowerBound + 1.0);

        return avg;
    }

    /**
     * Computes the average train standard deviation for the given interval
     *
     * @param ival The reference interval of accuracy points
     * @return the standard deviation of the train accuracy of the interval
     */
    public double intervalStdTrainAccuracy(Interval ival) {
        double std = 0;
        double avg = intervalMeanTrainAccuracy(ival);

        for (int i = ival.lowerBound; i <= ival.upperBound; i++) {
            std += accTrain[i] * accTrain[i];
        }
        std /= (double) (ival.upperBound - ival.lowerBound + 1);

        std -= Math.pow(avg, 2);

        std = Math.sqrt(std);
        return std;
    }

    /**
     * Computes the average train standard deviation for the given interval
     *
     * @param ival The reference interval of accuracy points
     * @return the standard deviation of the train accuracy of the interval
     */
    public double intervalStdTestAccuracy(Interval ival) {
        double std = 0;
        double avg = intervalMeanTestAccuracy(ival);

        for (int i = ival.lowerBound; i <= ival.upperBound; i++) {
            std += accTest[i] * accTest[i];
        }
        std /= (double) (ival.upperBound - ival.lowerBound + 1);

        std -= Math.pow(avg, 2);

        std = Math.sqrt(std);
        return std;
    }

    /**
     * Computes the average test accuracy of the given interval
     *
     * @param ival The reference interval of accuracy points
     * @return the mean test accuracy of the interval
     */
    public double intervalMeanTestAccuracy(Interval ival) {
        double avg = 0;

        for (int i = ival.lowerBound; i <= ival.upperBound; i++) {
            avg += accTest[i];
        }
        avg /= (double) (ival.upperBound - ival.lowerBound + 1.0);

        return avg;
    }

    /**
     * Computes the average difference between the training and test accuracy
     * values of the given interval
     *
     * @param ival The reference interval of accuracy points
     * @return the mean accuracy difference of the interval
     */
    public double intervalMeanTrainTestDifference(Interval ival) {
        double avg = 0;

        for (int i = ival.lowerBound; i <= ival.upperBound; i++) {
            avg += accTrain[i] - accTest[i];
        }
        avg /= (double) (ival.upperBound - ival.lowerBound + 1.0);

        return avg;
    }

    /**
     * Checks if any of the intervals in "ivals" are overlapped. If so, this
     * method try to merge them. If one interval contains another one, the
     * latter is dropped from the list
     *
     * @param ivals The list with the intervals
     * @param isQuality if we look for quality (true) intervals
     */
    public void mergeOverlappedIntervals(ArrayList<Interval> ivals, boolean isQuality) {
        Interval ival, ival2, newI, biggerIval;
        boolean change;


        //can we combine intervals separated by an small gap?
        //we adopt a greedy solution which merges always it can
        if (ivals.size() > 2) {
            for (int i = 0; i < ivals.size(); i++) {
                ival = ivals.get(i);
                change = false;
                ival2 = null;
                biggerIval = null;

                if (i + 1 < ivals.size()) {
                    ival2 = ivals.get(i + 1);
                }

                //interval to the right
                if (ival2 != null
                        && ival2.lowerBound - ival.upperBound < 5) {
                    if (isQuality
                            && isQualityInterval(new Interval(ival.lowerBound, ival2.upperBound))) {
                        biggerIval = new Interval(ival.lowerBound, ival2.upperBound);
                        change = true;
                    }
                    if (!isQuality
                            && isBadInterval(new Interval(ival.lowerBound, ival2.upperBound))) {
                        biggerIval = new Interval(ival.lowerBound, ival2.upperBound);
                        change = true;
                    }

                    if (change) {
                        ivals.remove(i + 1);
                        ivals.set(i, biggerIval);
                        //ivals.add(biggerIval);
                        i--;
                    }
                }

            }
        }

        //try to merge overlapped intervals
        for (int i = 0; i < ivals.size(); i++) {
            ival = ivals.get(i);
            change = false;
            for (int j = 0; j < ivals.size() && !change; j++) {
                if (i != j) {
                    ival2 = ivals.get(j);
                    //contains one the another?
                    if (ival.contains(ival2)) {
                        ivals.remove(j);
                        j = 0;
                    } //do they overlap?
                    else if (ival.overlap(ival2)) {
                        newI = new Interval();
                        newI.lowerBound = Math.min(ival.lowerBound, ival2.lowerBound);
                        newI.upperBound = Math.max(ival.upperBound, ival2.upperBound);

                        if (isQuality && isQualityInterval(newI)) {
                            ivals.set(i, newI);
                            ivals.remove(j);
                            change = true;
                        }
                        if (!isQuality && isBadInterval(newI)) {
                            ivals.set(i, newI);
                            ivals.remove(j);
                            change = true;
                        }
                    }

                }
            }
            //take again the changed -i- interval
            if (change) {
                i--;
            }
        }


        //drop those intervals below the minimum support (15% of the total of data sets)
        for (int i = 0; i < ivals.size(); i++) {
            if (ivals.get(i).upperBound - ivals.get(i).lowerBound + 1 < accTrain.length * minSupport) {
                ivals.remove(i);
                i--;
            }
        }
    }

    /**
     * Prints the interval in the standard output
     *
     * @param ival The interval to be printed
     */
    public void printInterval(Interval ival) {
        System.out.print("[" + ival.lowerCMet + "," + ival.upperCMet + "]");
        System.out.print("\t" + nf.format((double) 100.0 * (ival.length()) / (double) accTrain.length));
        System.out.print("\t" + nf.format(intervalMeanTrainAccuracy(ival)));
        System.out.print("\t" + nf.format((intervalMeanTrainAccuracy(ival) - globalTrainAvg)));
        System.out.print("\t" + nf.format(intervalMeanTestAccuracy(ival)));
        System.out.print("\t" + nf.format((intervalMeanTestAccuracy(ival) - globalTestAvg)));
        System.out.print("\t" + nf.format(intervalStdTrainAccuracy(ival)));
        System.out.print("\t" + nf.format(intervalStdTestAccuracy(ival)));
        System.out.print("\t" + nf.format((intervalMeanTrainAccuracy(ival) - globalTrainAvg)));
        System.out.print("\t" + nf.format((intervalMeanTestAccuracy(ival) - globalTestAvg)));
        System.out.print("\t" + nf.format(((intervalMeanTrainAccuracy(ival) - intervalMeanTestAccuracy(ival)) - globalDiff)));
        System.out.println();
    }

    protected void createDataFile(String filename) {

        Fichero.escribeFichero(filename, ""); //clear the file
        for (int i = 0; i < accTrain.length; i++) {
            Fichero.AnadirtoFichero(filename, i + "\t" + accTrain[i] + "\t" + accTest[i] + "\n");
        }
    }

    protected void plotInterval(String filename, String title, String data, int verticalLine1, int verticalLine2) {
        String script = "";
        script += "set terminal png\n";
        script += "set output \"" + filename + "\"\n";
        script += "set xlabel \"Data set\"\n";
        script += "set ylabel \"% Accuracy \"\n";
        script += "set title \"" + title + "\"\n";

        script += "set lmargin at screen 0.1\n";
        script += "set multiplot\n";
        script += "set xrange[0:" + dataset.length + "]\n";
        script += "plot \"" + data + "\" using 1:2 title \"Training accuracy\" smooth csplines with linespoints,"
                + "\"" + data + "\" using 1:3 title \"Test accuracy\" smooth csplines with linespoints\n";

        if (verticalLine2 != -1) {
            script += "set parametric\n";
            script += "set xrange[0:" + dataset.length + "]\n";
            script += "unset xlabel\n";
            script += "unset ylabel\n";
            script += "set xlabel \"\"\n";
            script += "set border 0\n";
            script += "set xtics nomirror\n";
            script += "set ytics nomirror\n";
            script += "set noxtics\n";
            script += "set noytics\n";
            script += "set key at graph 1, graph 0.9\n";
            script += "plot " + verticalLine2 + ",t title \"Upper bound = " + verticalLine2 + "\" with lines linecolor 4";
        }
        if (verticalLine1 != -1) {
            script += ", " + verticalLine1 + ",t  title \"Lower bound = " + verticalLine1 + "\" with lines linecolor 3";
        }
        script += "\n";

        Fichero.escribeFichero("script.gp", script);

        try {
            Process p2 = Runtime.getRuntime().exec("gnuplot script.gp");

            String line;
            BufferedReader input =
                    new BufferedReader(new InputStreamReader(p2.getInputStream()));
            while ((line = input.readLine()) != null) {
                //				System.out.println(line);
            }

            p2.waitFor();

            input.close();
            InputStream is = p2.getInputStream();
            InputStream es = p2.getErrorStream();
            OutputStream os = p2.getOutputStream();
            //Do something with the proc or inputstream
            is.close();
            is = null;
            es.close();
            es = null;
            os.close();
            os = null;
            p2.destroy();

        } catch (Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }

    }

    protected void plotIntervalsPNG(String filename, String title, String data, ArrayList<Interval> bounds) {
        int color;
        String figtitle;


        String script = "";
        script += "set terminal png\n";
        script += "set output \"" + filename + "\"\n";
        script += "set xlabel \"Data set\"\n";
        script += "set ylabel \"% Accuracy \"\n";
        script += "set title \"" + title + "\"\n";
        script += "set key font \",40\"\n";

        script += "set lmargin at screen 0.1\n";
        script += "set multiplot\n";
        script += "set xrange[0:" + dataset.length + "]\n";
        script += "plot \"" + data + "\" using 1:2 title \"Training accuracy\" smooth csplines with linespoints lw 2,"
                + "\"" + data + "\" using 1:3 title \"Test accuracy\" smooth csplines with linespoints lw 2\n";

        for (int i = 0; i < bounds.size(); i++) {
            if (bounds.get(i).goodInterval) {
                color = 3;
                figtitle = "Good Behavior";
            } else {
                color = 7;
                figtitle = "Bad Behavior";
            }
            if (i == 0) {
                script += "set parametric\n";
                script += "set xrange[0:" + dataset.length + "]\n";
                script += "unset xlabel\n";
                script += "unset ylabel\n";
                script += "set xlabel \"\"\n";
                script += "set border 0\n";
                script += "set xtics nomirror\n";
                script += "set ytics nomirror\n";
                script += "set noxtics\n";
                script += "set noytics\n";
                script += "set key at graph 1, graph 0.2\n";

                script += "plot " + bounds.get(i).lowerBound + ",t  title \"" + figtitle + " Lower bound = " + nf.format(bounds.get(i).lowerCMet) + "\" with lines lw 2 linecolor " + color;
                if (bounds.get(i).upperCMet != Double.POSITIVE_INFINITY) {
                    script += ", " + bounds.get(i).upperBound + ",t title \"" + figtitle + " Upper bound = " + nf.format(bounds.get(i).upperCMet) + "\" with lines lw 2 linecolor " + color;
                }

            } else {
                script += ", " + bounds.get(i).lowerBound + ",t  title \"" + figtitle + " Lower bound = " + nf.format(bounds.get(i).lowerCMet) + "\" with lines lw 2 linecolor " + color;
                if (bounds.get(i).upperCMet != Double.POSITIVE_INFINITY) {
                    script += ", " + bounds.get(i).upperBound + ",t  title \"" + figtitle + " Upper bound = " + nf.format(bounds.get(i).upperCMet) + "\" with lines lw 2 linecolor " + color;
                }
            }
        }
        script += "\n";

        Fichero.escribeFichero("script.gp", script);

        try {
            Process p2 = Runtime.getRuntime().exec("gnuplot script.gp");

            String line;
            BufferedReader input =
                    new BufferedReader(new InputStreamReader(p2.getInputStream()));
            while ((line = input.readLine()) != null) {
                //				System.out.println(line);
            }

            p2.waitFor();

            input.close();
            InputStream is = p2.getInputStream();
            InputStream es = p2.getErrorStream();
            OutputStream os = p2.getOutputStream();
            //Do something with the proc or inputstream
            is.close();
            is = null;
            es.close();
            es = null;
            os.close();
            os = null;
            p2.destroy();

        } catch (Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }

    }

    protected void plotIntervalsEPS_color(String filename, String title, String data, ArrayList<Interval> bounds) {
        String color;
        String figtitle;


        String script = "";
        script += "set terminal postscript eps enhanced color font 'Helvetica,20' linewidth 3 dl 6\n";
        script += "set output \"" + filename + "\"\n";
        script += "set size 2.5,2.5\n";
        script += "set xlabel \"Data set\"\n";
        script += "set xlabel font \",50\"\n";
        script += "set ylabel \"% Accuracy \"\n";
        script += "set ylabel font \",50\"\n";
        script += "set title \"" + title + "\"\n";
        script += "set key font \",50\"\n";
        script += "set key spacing 6\n";

        script += "set lmargin at screen 0.1\n";
        script += "set multiplot\n";
        script += "set size 2.5,2.5\n";
        script += "set xrange[0:" + dataset.length + "]\n";
        script += "plot \"" + data + "\" using 1:2 title \"Training accuracy\" smooth csplines with linespoints lw 2 lt 3 lc 1,"
                + "\"" + data + "\" using 1:3 title \"Test accuracy\" smooth csplines with linespoints lw 2 lt 1 lc 2\n";

        for (int i = 0; i < bounds.size(); i++) {
            if (bounds.get(i).goodInterval) {
                color = "rgb \"blue\"";
                figtitle = "Good Behavior";
            } else {
                color = "rgb \"orange\"";;
                figtitle = "Bad Behavior";
            }
            if (i == 0) {
                script += "set parametric\n";
                script += "set xrange[0:" + dataset.length + "]\n";
                script += "unset xlabel\n";
                script += "unset ylabel\n";
                script += "set xlabel \"\"\n";
                script += "set border 0\n";
                script += "set xtics nomirror\n";
                script += "set ytics nomirror\n";
                script += "set noxtics\n";
                script += "set noytics\n";
                script += "set key at graph 1, graph 0.2\n";

                script += "plot " + bounds.get(i).lowerBound + ",t  title \"" + figtitle + " Lower bound = " + nf.format(bounds.get(i).lowerCMet) + "\" with lines lt 1 lw 5 linecolor " + color;
                if (bounds.get(i).upperCMet != Double.POSITIVE_INFINITY) {
                    script += ", " + bounds.get(i).upperBound + ",t title \"" + figtitle + " Upper bound = " + nf.format(bounds.get(i).upperCMet) + "\" with lines lt 1 lw 5 linecolor " + color;
                }

            } else {
                script += ", " + bounds.get(i).lowerBound + ",t  title \"" + figtitle + " Lower bound = " + nf.format(bounds.get(i).lowerCMet) + "\" with lines lt 1 lw 5 linecolor " + color;
                if (bounds.get(i).upperCMet != Double.POSITIVE_INFINITY) {
                    script += ", " + bounds.get(i).upperBound + ",t  title \"" + figtitle + " Upper bound = " + nf.format(bounds.get(i).upperCMet) + "\" with lines lt 1 lw 5 linecolor " + color;
                }
            }
        }
        script += "\n";

        Fichero.escribeFichero("script.gp", script);

        try {
            Process p2 = Runtime.getRuntime().exec("gnuplot script.gp");

            String line;
            BufferedReader input =
                    new BufferedReader(new InputStreamReader(p2.getInputStream()));
            while ((line = input.readLine()) != null) {
                //				System.out.println(line);
            }

            p2.waitFor();

            input.close();
            InputStream is = p2.getInputStream();
            InputStream es = p2.getErrorStream();
            OutputStream os = p2.getOutputStream();
            //Do something with the proc or inputstream
            is.close();
            is = null;
            es.close();
            es = null;
            os.close();
            os = null;
            p2.destroy();

        } catch (Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }

    }

    protected void plotIntervalsEPS_bn(String filename, String title, String data, ArrayList<Interval> bounds) {
        String color;
        String figtitle;


        String script = "";
        script += "set terminal postscript eps enhanced monochrome font 'Helvetica,20' linewidth 3 dl 6\n";
        script += "set output \"" + filename + "\"\n";
        script += "set size 2.5,2.5\n";
        script += "set xlabel \"Data set\"\n";
        script += "set ylabel \"% Accuracy \"\n";
        script += "set title \"" + title + "\"\n";
        script += "set key font \",50\"\n";
        script += "set key spacing 6\n";

        script += "set lmargin at screen 0.1\n";
        script += "set multiplot\n";
        script += "set size 2.5,2.5\n";
        script += "set xrange[0:" + dataset.length + "]\n";
        script += "plot \"" + data + "\" using 1:2 title \"Training accuracy\" smooth csplines with linespoints lw 2 lt 3 lc 1,"
                + "\"" + data + "\" using 1:3 title \"Test accuracy\" smooth csplines with linespoints lw 2 lt 1 lc 2\n";

        for (int i = 0; i < bounds.size(); i++) {
            if (bounds.get(i).goodInterval) {
                color = "rgb \"blue\"";
                figtitle = "Good Behavior";
            } else {
                color = "rgb \"orange\"";;
                figtitle = "Bad Behavior";
            }
            if (i == 0) {
                script += "set parametric\n";
                script += "set xrange[0:" + dataset.length + "]\n";
                script += "unset xlabel\n";
                script += "unset ylabel\n";
                script += "set xlabel \"\"\n";
                script += "set border 0\n";
                script += "set xtics nomirror\n";
                script += "set ytics nomirror\n";
                script += "set noxtics\n";
                script += "set noytics\n";
                script += "set key at graph 1, graph 0.2\n";

                script += "plot " + bounds.get(i).lowerBound + ",t  title \"" + figtitle + " Lower bound = " + nf.format(bounds.get(i).lowerCMet) + "\" with lines linecolor " + color;

                script += ", " + bounds.get(i).upperBound + ",t title \"" + figtitle + " Upper bound = " + nf.format(bounds.get(i).upperCMet) + "\" with lines linecolor " + color;

            } else {
                script += ", " + bounds.get(i).lowerBound + ",t  title \"" + figtitle + " Lower bound = " + nf.format(bounds.get(i).lowerCMet) + "\" with lines linecolor " + color;
                script += ", " + bounds.get(i).upperBound + ",t  title \"" + figtitle + " Upper bound = " + nf.format(bounds.get(i).upperCMet) + "\" with lines linecolor " + color;
            }
        }
        script += "\n";

        Fichero.escribeFichero("script.gp", script);

        try {
            Process p2 = Runtime.getRuntime().exec("gnuplot script.gp");

            String line;
            BufferedReader input =
                    new BufferedReader(new InputStreamReader(p2.getInputStream()));
            while ((line = input.readLine()) != null) {
                //				System.out.println(line);
            }

            p2.waitFor();

            input.close();
            InputStream is = p2.getInputStream();
            InputStream es = p2.getErrorStream();
            OutputStream os = p2.getOutputStream();
            //Do something with the proc or inputstream
            is.close();
            is = null;
            es.close();
            es = null;
            os.close();
            os = null;
            p2.destroy();

        } catch (Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }

    }
}
