/*
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

/*
 *    IntervalExtract.java
 *    Copyright (C) 2013 Julian Luengo
 *
 */
package complexityruleextraction;

import static java.util.Arrays.*;
import static java.io.File.*;
import java.util.List;
import joptsimple.*;

/**
 *
 * @author Julian Luengo Martin
 * @date 2008
 */
public class Main {

    public static void printHelp() {
        System.err.println();

        System.err.println("Example of usage:");
        System.err.println();
        System.err.println("One file mode:");
        System.err.println("--------------");
        System.err.println("-t <text file with the complexity measures+accuracy> -h <threhold value> -m <minimum accuracy value>");
        System.err.println();

        System.err.println("List of files mode (deprecated) :");
        System.err.println("---------------------------------");
        System.err.println("-l <list to the files with the complexity measures+accuracy>  -h <threhold value> -m <minimum accuracy value>");
        System.err.println();

        System.err.println("Validation mode:");
        System.err.println("----------------");
        System.err.println("-v <text file with the intervals as printed by -t option>;<text file with the complexity measures+accuracy of the validation data sets> (WINDOWS MODE)");
        System.err.println("-v <text file with the intervals as printed by -t option>:<text file with the complexity measures+accuracy of the validation data sets> (LINUX MODE)");
    }

    public static void main(String args[]) throws Exception {
        int optionUsed;
        double threshold, minTest;

        OptionParser parser = new OptionParser() {
            {
                accepts("t", "One file mode").withRequiredArg()
                        .ofType(String.class)
                        .describedAs("Text File in \"path\"");
                accepts("l", "List of files mode").withRequiredArg()
                        .ofType(String.class)
                        .describedAs("Text File in \"path\"");
                accepts("v", "Validation mode with Text file with intervals in \"path 1\" and text file with the complexity measures+accuracy of the validation data sets in \"path 2\"")
                        .withRequiredArg()
                        .ofType(String.class)
                        .withValuesSeparatedBy(pathSeparatorChar )
                        .describedAs("<Text file in \"path 1\"> <Text file in \"path 2\">");
                accepts("?", "prints help")
                        .forHelp();
                accepts("epscolor", "Print color EPS figures using GNUplot");
                accepts("epsbn", "Print black and white EPS figures using GNUplot");
                accepts("png", "Print color PNG figures using GNUplot");
                acceptsAll(asList("threshold", "h"), "Threshold used to find intervals of good or bad behavior")
                        .requiredIf("l")
                        .requiredIf("t")
                        .withRequiredArg()
                        .ofType(Double.class)
                        .describedAs("threshold")
                        .defaultsTo(8.5);
                acceptsAll(asList("minGoodElementTest", "m"), "Minimum test accuracy value needed to be a good behavior element")
                        .requiredIf("l")
                        .requiredIf("t")
                        .withRequiredArg()
                        .ofType(Double.class)
                        .describedAs("threshold")
                        .defaultsTo(90.0);

            }
        };

        try {
            OptionSet options = parser.parse(args);

            if (options.has("?")) {
                parser.printHelpOn(System.err);
                printHelp();
            } else {

                //only one mode can be used at once. Check it
                optionUsed = 0;
                if (options.has("t")) {
                    optionUsed++;
                }
                if (options.has("l")) {
                    optionUsed++;
                }
                if (options.has("v")) {
                    optionUsed++;
                }
                if (optionUsed != 1) {
                    System.err.println("Error: one mode (and only one) must be used at once: 'l', 't' or 'v'");
                    System.err.println();
                    parser.printHelpOn(System.err);
                    printHelp();
                } else {
                    if (options.has("l")) {
                        threshold = (Double)options.valueOf("h");
                        minTest = (Double) options.valueOf("m");
                        IntervalExtract iExt = new IntervalExtract(threshold, minTest, options.has("epscolor"), options.has("epsbn"), options.has("png"));
                        iExt.processList((String) options.valueOf("l"));
                    } else if (options.has("t")) {
                        threshold = (Double)options.valueOf("h");
                        minTest = (Double) options.valueOf("m");
                        IntervalExtract iExt = new IntervalExtract(threshold, minTest, options.has("epscolor"), options.has("epsbn"), options.has("png"));
                        iExt.processTable((String) options.valueOf("t"));
                    } else if (options.has("v")) {
                        if(options.valuesOf("v").size()!=2){
                            System.err.println("Error. Specify 2 filepaths for the '-v' option separated by ; or : (Win/Linux respectively).");
                            System.err.println();
                            printHelp();
                            System.exit(1);
                        }else{
                            IntervalValidation iValidate = new IntervalValidation();
                            iValidate.readIntervals(((List<String>) options.valuesOf("v")).get(0));
                            iValidate.matchIntervals(((List<String>) options.valuesOf("v")).get(1));
                        }
                    }
                }
            }
        } catch (OptionException e) {
            System.err.println("Error found. Check option(s) " + e.options() + " requeriments.");
            parser.printHelpOn(System.err);
            printHelp();
            System.exit(1);
        }


    }
}
